Package: markdown
Type: Package
Title: Render Markdown with 'commonmark'
Version: 2.0
Authors@R: c(
   person("Yihui", "Xie", role = c("aut", "cre"), email = "xie@yihui.name", comment = c(ORCID = "0000-0003-0645-5666")),
   person("JJ", "Allaire", role = "aut"),
   person("Jeffrey", "Horner", role = "aut"),
   person("Henrik", "Bengtsson", role = "ctb"),
   person("Jim", "Hester", role = "ctb"),
   person("Yixuan", "Qiu", role = "ctb"),
   person("Kohske", "Takahashi", role = "ctb"),
   person("Adam", "November", role = "ctb"),
   person("Nacho", "Caballero", role = "ctb"),
   person("Jeroen", "Ooms", role = "ctb"),
   person("Thomas", "Leeper", role = "ctb"),
   person("Joe", "Cheng", role = "ctb"),
   person("Andrzej", "Oles", role = "ctb"),
   person(given = "Posit Software, PBC", role = c("cph", "fnd"))
   )
Description: Render Markdown to full and lightweight HTML/LaTeX documents with
    the 'commonmark' package. This package has been superseded by 'litedown'.
Depends: R (>= 2.11.1)
Imports: utils, xfun, litedown (>= 0.6)
Suggests: knitr, rmarkdown (>= 2.18), yaml, RCurl
License: MIT + file LICENSE
URL: https://github.com/rstudio/markdown
BugReports: https://github.com/rstudio/markdown/issues
RoxygenNote: 7.3.2
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2025-03-23 17:06:26 UTC; runner
Author: Yihui Xie [aut, cre] (<https://orcid.org/0000-0003-0645-5666>),
  JJ Allaire [aut],
  Jeffrey Horner [aut],
  Henrik Bengtsson [ctb],
  Jim Hester [ctb],
  Yixuan Qiu [ctb],
  Kohske Takahashi [ctb],
  Adam November [ctb],
  Nacho Caballero [ctb],
  Jeroen Ooms [ctb],
  Thomas Leeper [ctb],
  Joe Cheng [ctb],
  Andrzej Oles [ctb],
  Posit Software, PBC [cph, fnd]
Maintainer: Yihui Xie <xie@yihui.name>
Repository: CRAN
Date/Publication: 2025-03-23 19:30:03 UTC
Built: R 4.6.0; ; 2026-04-25 23:18:24 UTC; unix
