# Generated by roxygen2: do not edit by hand

export(html_format)
export(latex_format)
export(mark)
export(mark_html)
export(mark_latex)
export(markdownToHTML)
export(markdown_options)
export(renderMarkdown)
export(rpubsUpload)
export(smartypants)
import(utils)
