# Markdown

[![Build](https://github.com/AdaCore/markdown/workflows/Build/badge.svg)](https://github.com/AdaCore/markdown/actions)
[![codecov](https://codecov.io/gh/AdaCore/markdown/branch/master/graph/badge.svg)](https://codecov.io/gh/AdaCore/markdown)

> Markdown parser in Ada

This library provides a markdown parser written in Ada.

## Install

### Build from sources

Preferred way to install is to download sources and run

    make all install PREFIX=/path/to/install

### Dependencies

This code requires the [VSS library](https://github.com/AdaCore/VSS).

## Maintainers

[@AdaCore](https://adacore.com/).

## Contribute

Feel free to dive in!
[Open an issue](https://github.com/AdaCore/markdown/issues/new)
or submit pull requests.

## License

[Apache-2.0](LICENSE).
