#!/bin/sh

set -eu

# The readme_control.html file has been generated with
# SOURCE_DATE_EPOCH=1723582475

SOURCE_DATE_EPOCH=1723582475 markdown 'Markdown Readme.text' > "$AUTOPKGTEST_TMP/readme_new.html"
exec diff 'debian/tests/readme_control.html' "$AUTOPKGTEST_TMP/readme_new.html"
