<!--
%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Some Markdown Examples}
-->

This vignette shows some examples for different Markdown extensions and HTML options.

```{r read-code, include=FALSE}
library(knitr)
opts_chunk$set(tidy = FALSE)  # otherwise \n will cause problems
read_chunk(system.file('examples', 'markdownExtensions.R', package = 'markdown'),
           labels = 'md-extensions')
read_chunk(system.file('examples', 'HTMLOptions.R', package = 'markdown'),
           labels = 'html-options')
```

```{r}
library(markdown)
```

# Markdown Extensions

```{r md-extensions}
```

# HTML Options

```{r html-options}
```


```{r include=FALSE}
options(markdown.HTML.options=markdownHTMLOptions(defaults=TRUE))
```
