title: Installation

# Installing Python-Markdown

## The Easy Way

The easiest way to install Python-Markdown is simply to type the
following command from the command line:

```bash
pip install markdown
```

That's it! You're ready to [use](reference.md) Python-Markdown. Enjoy!

For more detailed instructions on installing Python packages, see the
[Installing Packages] tutorial in the [Python Packaging User Guide].

[Installing Packages]: https://packaging.python.org/tutorials/installing-packages/
[Python Packaging User Guide]: https://packaging.python.org/

## Using the Git Repository {: #git }

If you're the type that likes to live on the edge, you may want to keep up with
the latest additions and bug fixes in the repository between releases.
Python-Markdown is maintained in a Git repository on GitHub.com. To
get a copy of Python-Markdown from the repository do the following from the
command line:

```bash
pip install git+https://github.com/Python-Markdown/markdown.git
```
