<!--
%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{HTML Output from Markdown Examples}
-->

This vignette shows how the output looks like in HTML from the other vignette `markdown-examples.Rmd`, in which the literal HTML code was shown.

```{r read, include=FALSE}
library(knitr)
opts_chunk$set(results = 'asis')
out = knit_child(text = scan('markdown-examples.Rmd', what = 'character', skip = 7, sep = '\n'))
```

`r paste(out, collapse = '\n')`
