---
title: Raw Output from Markdown Examples
---

<!--
%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Some Markdown Examples}
-->

This vignette shows some examples for different Markdown rendering options.

```{r render-options, tidy=FALSE, file=markdown:::pkg_file('examples', 'render-options.R'), comment=''}
```
